@@ -79,7 +79,7 @@ class CodeGen:
             vars_ += map(lambda x: x.py_name, context.input_vars)
             vars_ += map(lambda x: x.ver_name, context.input_vars)
             vars_ += map(lambda x: x.ver_name, context.output_vars)
-            vars_ += map(lambda x: x.ver_name, context.global_vars)
+            vars_ += map(lambda x: x.ver_name, context.local_vars)
             str_ = f'$display("{context.name},%s,'
             str_ += "=%0d,".join(vars_) + '=%0d", '
             str_ += f"{context.state_var.ver_name}.name, "
@@ -91,7 +91,7 @@ class CodeGen:
             """
             Instance signals that should always be set to zero be default
             """
-            for instance in context.instances.values():
+            for instance in context.generator_instances.values():
                 yield ver.NonBlockingSubsitution(instance.signals.ready, ir.UInt(0))
                 yield ver.NonBlockingSubsitution(instance.signals.start, ir.UInt(0))
 
@@ -101,7 +101,6 @@ class CodeGen:
                 ver.Statement("`ifdef DEBUG"),
                 ver.Statement(make_debug_display(context)),
                 ver.Statement("`endif"),
-                ver.NonBlockingSubsitution(context.signals.done, ir.UInt(0)),
                 ver.Statement(),
             ]
             + list(create_instance_zeroed_signals())
@@ -111,7 +110,14 @@ class CodeGen:
                     context.signals.ready,
                     cast(
                         list[ver.Statement],
-                        [ver.NonBlockingSubsitution(context.signals.valid, ir.UInt(0))],
+                        [
+                            ver.NonBlockingSubsitution(
+                                context.signals.valid, ir.UInt(0)
+                            ),
+                            ver.NonBlockingSubsitution(
+                                context.signals.done, ir.UInt(0)
+                            ),
+                        ],
                     ),
                     [],
                 ),
@@ -120,11 +126,19 @@ class CodeGen:
                 ver.Statement(),
                 ver.Statement(comment="Start signal takes precedence over reset"),
                 ver.IfElse(
-                    context.signals.reset,
+                    ir.UBinOp(context.signals.reset, "||", context.signals.start),
                     then_body=[
                         ver.NonBlockingSubsitution(
-                            lvalue=context.state_var,
-                            rvalue=context.idle_state,
+                            context.state_var,
+                            context.idle_state,
+                        ),
+                        ver.NonBlockingSubsitution(
+                            context.signals.done,
+                            ir.UInt(0),
+                        ),
+                        ver.NonBlockingSubsitution(
+                            context.signals.valid,
+                            ir.UInt(0),
                         ),
                     ],
                     else_body=[],
@@ -136,13 +150,12 @@ class CodeGen:
         body: list[ver.Statement] = []
 
         body += [
-            ver.Statement(comment="Global variables"),
+            ver.Statement(comment="Local variables"),
         ]
-
-        context.global_vars = sorted(context.global_vars, key=lambda x: x.ver_name)
+        context.local_vars.sort(key=lambda x: x.ver_name)
         body += [
             ver.Declaration(v.ver_name, reg=True, signed=True)
-            for v in context.global_vars
+            for v in context.local_vars
         ]
 
         body += [
@@ -150,7 +163,7 @@ class CodeGen:
             for var in context.input_vars
         ]
 
-        for instance in context.instances.values():
+        for instance in context.generator_instances.values():
             body.append(
                 ver.Statement(
                     comment="================ Function Instance ================"
@@ -272,7 +285,7 @@ class CodeGen:
             )
 
         if_else = ver.IfElse(
-            ir.Expression("_start"),
+            context.signals.start,
             then_body,
             [
                 ver.Statement(
@@ -289,7 +302,6 @@ class CodeGen:
                 ),
             ],
         )
-        # logging.debug(f"make start if else {if_else}")
         return [if_else]
 
     @property
@@ -371,6 +383,7 @@ class CodeGen:
         initial_body.append(ver.BlockingSub(self.context.signals.reset, ir.UInt(0)))
         initial_body.append(ver.Statement())
 
+        logging.debug("Making test cases")
         for i, test_case in enumerate(self.context.test_cases):
             # New test case and start
             initial_body.append(
@@ -404,22 +417,17 @@ class CodeGen:
 
             # While loop waitng for ready signal
             while_body: list[ver.Statement] = []
-            # while_body.append(make_display_stmt())
-            if config.random_ready:
-                while_body.append(ver.Statement("_ready = $urandom_range(0, 4) === 0;"))
-            while_body.append(
-                ver.Statement(
-                    comment="`if (_ready && _valid)` also works as a conditional"
-                )
-            )
+            while_body.append(ver.AtPosedgeStatement(self.context.signals.clock))
             while_body.append(
                 ver.IfElse(
-                    condition=ir.Expression("_ready"),
+                    condition=self.context.signals.ready,
                     then_body=[make_display_stmt()],
                     else_body=[],
                 )
             )
             while_body.append(ver.AtNegedgeStatement(self.context.signals.clock))
+            if config.random_ready:
+                while_body.append(ver.Statement("_ready = $urandom_range(0, 4) === 0;"))
 
             initial_body.append(
                 ver.While(
@@ -431,19 +439,20 @@ class CodeGen:
                     body=while_body,
                 )
             )
-            initial_body.append(
-                ver.IfElse(
-                    condition=ir.Expression("_ready"),
-                    then_body=[make_display_stmt()],
-                    else_body=[],
-                )
-            )
+            # initial_body.append(
+            #     ver.IfElse(
+            #         condition=self.context.signals.ready,
+            #         then_body=[make_display_stmt()],
+            #         else_body=[],
+            #     )
+            # )
             initial_body.append(ver.Statement())
 
         initial_body.append(ver.Statement(literal="$finish;"))
 
         initial_loop = ver.Initial(body=initial_body)
 
+        logging.debug("Creating python test code")
         python_test_code = Lines()
         for case in self.context.test_cases:
             python_test_code += f"print(list({self.context.name}(*{case})))"
@@ -508,17 +517,6 @@ class CaseBuilder:
         # Reverse states for readability (states are built backwards)
         self.case.case_items = list(reversed(self.case.case_items))
 
-        # Add done state if it doesn't exist in cases
-        if all(
-            case.condition != self.context.done_state for case in self.case.case_items
-        ):
-            self.case.case_items.append(
-                ver.CaseItem(
-                    condition=self.context.done_state,
-                    statements=[self.create_quick_done(self.context)],
-                )
-            )
-
         return self.case
 
     @staticmethod
@@ -534,7 +532,6 @@ class CaseBuilder:
             condition=ir.UBinOp(
                 ir.UnaryOp("!", context.signals.valid), "&&", context.signals.ready
             ),
-            # condition=context.signals.ready,
             then_body=[
                 ver.NonBlockingSubsitution(context.signals.done, ir.UInt(1)),
                 ver.NonBlockingSubsitution(context.state_var, context.idle_state),
@@ -565,10 +562,7 @@ class CaseBuilder:
 
         stmts: list[ver.Statement] = []
 
-        if isinstance(vertex, ir.DoneNode):
-            stmts.append(self.create_quick_done(self.context))
-
-        elif isinstance(vertex, ir.AssignNode):
+        if isinstance(vertex, ir.AssignNode):
             stmts.append(
                 ver.NonBlockingSubsitution(
                     vertex.lvalue,
@@ -608,4 +602,6 @@ class CaseBuilder:
                     self.context.state_var, ir.State(edge.optimal_child.unique_id)
                 )
             ]
+        if isinstance(edge, type(None)):
+            return []
         raise RuntimeError(f"{type(edge)}")
